/*
 * Catch.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using Eagle._Attributes;
using Eagle._Components.Private;
using Eagle._Components.Public;
using Eagle._Containers.Public;
using Eagle._Interfaces.Public;

namespace Eagle._Commands
{
    [ObjectId("382b9037-4351-47c7-a9a6-e66bcbfe284d")]
    [CommandFlags(CommandFlags.Safe | CommandFlags.Standard)]
    [ObjectGroup("control")]
    internal sealed class Catch : Core
    {
        public Catch(
            ICommandData commandData
            )
            : base(commandData)
        {
            // do nothing.
        }

        #region IExecute Members
        public override ReturnCode Execute(
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            ReturnCode code;

            if (interpreter != null)
            {
                if (arguments != null)
                {
                    if ((arguments.Count == 2) || (arguments.Count == 3))
                    {
                        string name = StringList.MakeList("catch");

                        ICallFrame frame = interpreter.NewLinkedCallFrame(
                            name, null, CallFrameFlags.Catch | CallFrameFlags.Tracking);

                        interpreter.PushCallFrame(frame);

                        EngineFlags engineFlags = interpreter.EngineFlags;

                        if (Engine.HasNoResetError(engineFlags))
                            engineFlags |= EngineFlags.ErrorAlreadyLogged;

                        Result localResult = null;

                        code = interpreter.EvaluateScript(arguments[1], engineFlags, ref localResult);

                        //
                        // BUGFIX: Prevent messing up custom errorInfo from the [error] 
                        //         command by checking to see if the "error already logged" 
                        //         flag has been set.
                        //
                        engineFlags = interpreter.EngineFlags;

                        if ((code == ReturnCode.Error) &&
                            !Engine.HasErrorAlreadyLogged(engineFlags) &&
                            !Engine.HasNoResetError(engineFlags))
                        {
                            Engine.AddErrorInformation(interpreter, localResult,
                                String.Format("{0}    (\"catch\" body line {1})",
                                    Environment.NewLine, interpreter.ErrorLine));
                        }

                        //
                        // NOTE: Pop the original call frame that we pushed above and 
                        //       any intervening scope call frames that may be leftover 
                        //       (i.e. they were not explicitly closed).
                        //
                        interpreter.PopCallFrame(CallFrameFlags.Scope, true);

                        Engine.ResetResult(interpreter, ref result);
                        result = ConversionOps.ToInt(code);

                        if (arguments.Count == 3)
                        {
                            Result error = null;

                            code = interpreter.SetVariableValue(
                                VariableFlags.NoReady, arguments[2], localResult, null,
                                ref error);

                            if (code != ReturnCode.Ok)
                            {
                                Engine.ResetResult(interpreter, ref result);

                                result = String.Format(
                                    "couldn't save command result in variable: {0}",
                                    error);
                            }
                        }
                        else
                        {
                            //
                            // NOTE: We are "catching" (masking) the error; therefore,
                            //       do not propogate it.
                            //
                            code = ReturnCode.Ok;
                        }
                    }
                    else
                    {
                        result = "wrong # args: should be \"catch script ?varName?\"";
                        code = ReturnCode.Error;
                    }
                }
                else
                {
                    result = "invalid argument list";
                    code = ReturnCode.Error;
                }
            }
            else
            {
                result = "invalid interpreter";
                code = ReturnCode.Error;
            }

            return code;
        }
        #endregion
    }
}
