/*
 * Configuration.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Security.Cryptography.X509Certificates;
using Eagle._Components.Private.Delegates;

namespace Eagle._Components.Private
{
    [Guid("75620dd2-d59d-4cf0-87ff-5ecad2472bd2")]
    internal sealed class Configuration
    {
        #region Private Constants
        private static readonly string TraceCategory = typeof(Configuration).Name;
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Private Data
        private bool isAuthenticodeSigned;
        private bool isStrongNameSigned;
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Private Constructors
        private Configuration()
        {
            // do nothing.
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Public Constructors
        public Configuration(
            Assembly assembly,
            string subjectName,
            X509Certificate2 certificate2,
            string protocolId,
            byte[] publicKeyToken,
            int delay,
            string mutexName,
            Uri baseUri,
            string tagFileName,
            string uriFormat,
            string name,
            CultureInfo culture,
            Version patchLevel,
            BuildType buildType,
            ReleaseType releaseType,
            SignatureFlags signatureFlags,
            string coreDirectory,
            string coreFileName,
            string hashAlgorithm,
            string commandFormat,
            string argumentFormat,
            string logFileName,
            TraceCallback traceCallback,
            string[] shellArgs,
            bool coreIsAssembly,
            bool whatIf,
            bool verbose,
            bool silent,
            bool invisible,
            bool force,
            bool tracing,
            bool logging,
            bool shell
            )
            : this()
        {
            this.assembly = assembly;
            this.subjectName = subjectName;
            this.certificate2 = certificate2;
            this.protocolId = protocolId;
            this.publicKeyToken = publicKeyToken;
            this.delay = delay;
            this.mutexName = mutexName;
            this.baseUri = baseUri;
            this.tagFileName = tagFileName;
            this.uriFormat = uriFormat;
            this.name = name;
            this.culture = culture;
            this.patchLevel = patchLevel;
            this.buildType = buildType;
            this.releaseType = releaseType;
            this.signatureFlags = signatureFlags;
            this.coreDirectory = coreDirectory;
            this.coreFileName = coreFileName;
            this.hashAlgorithm = hashAlgorithm;
            this.commandFormat = commandFormat;
            this.argumentFormat = argumentFormat;
            this.logFileName = logFileName;
            this.traceCallback = traceCallback;
            this.shellArgs = shellArgs;
            this.coreIsAssembly = coreIsAssembly;
            this.whatIf = whatIf;
            this.verbose = verbose;
            this.silent = silent;
            this.invisible = invisible;
            this.force = force;
            this.tracing = tracing;
            this.logging = logging;
            this.shell = shell;
        }

        ///////////////////////////////////////////////////////////////////////

        public Configuration(
            Configuration configuration
            )
            : this()
        {
            if (configuration != null)
            {
                this.assembly = configuration.assembly;
                this.subjectName = configuration.subjectName;
                this.certificate2 = configuration.certificate2;
                this.protocolId = configuration.protocolId;
                this.publicKeyToken = configuration.publicKeyToken;
                this.mutexName = configuration.mutexName;
                this.baseUri = configuration.baseUri;
                this.tagFileName = configuration.tagFileName;
                this.uriFormat = configuration.uriFormat;
                this.name = configuration.name;
                this.culture = configuration.culture;
                this.patchLevel = configuration.patchLevel;
                this.buildType = configuration.buildType;
                this.releaseType = configuration.releaseType;
                this.signatureFlags = configuration.signatureFlags;
                this.coreDirectory = configuration.coreDirectory;
                this.coreFileName = configuration.coreFileName;
                this.hashAlgorithm = configuration.hashAlgorithm;
                this.commandFormat = configuration.commandFormat;
                this.argumentFormat = configuration.argumentFormat;
                this.logFileName = configuration.logFileName;
                this.traceCallback = configuration.traceCallback;
                this.shellArgs = configuration.shellArgs;
                this.coreIsAssembly = configuration.coreIsAssembly;
                this.whatIf = configuration.whatIf;
                this.verbose = configuration.verbose;
                this.silent = configuration.silent;
                this.invisible = configuration.invisible;
                this.force = configuration.force;
                this.tracing = configuration.tracing;
                this.logging = configuration.logging;
                this.shell = configuration.shell;
            }
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Public Properties
        private Assembly assembly;
        public Assembly Assembly
        {
            get { return assembly; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string subjectName;
        public string SubjectName
        {
            get { return subjectName; }
        }

        ///////////////////////////////////////////////////////////////////////

        private X509Certificate2 certificate2;
        public X509Certificate2 Certificate2
        {
            get { return certificate2; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string protocolId;
        public string ProtocolId
        {
            get { return protocolId; }
        }

        ///////////////////////////////////////////////////////////////////////

        private byte[] publicKeyToken;
        public byte[] PublicKeyToken
        {
            get { return publicKeyToken; }
        }

        ///////////////////////////////////////////////////////////////////////

        private int delay;
        public int Delay
        {
            get { return delay; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string mutexName;
        public string MutexName
        {
            get { return mutexName; }
        }

        ///////////////////////////////////////////////////////////////////////

        private Uri baseUri;
        public Uri BaseUri
        {
            get { return baseUri; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string tagFileName;
        public string TagFileName
        {
            get { return tagFileName; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string uriFormat;
        public string UriFormat
        {
            get { return uriFormat; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string name;
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private CultureInfo culture;
        public CultureInfo Culture
        {
            get { return culture; }
            set { culture = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private Version patchLevel;
        public Version PatchLevel
        {
            get { return patchLevel; }
        }

        ///////////////////////////////////////////////////////////////////////

        private BuildType buildType;
        public BuildType BuildType 
        {
            get { return buildType; }
        }

        ///////////////////////////////////////////////////////////////////////

        private ReleaseType releaseType;
        public ReleaseType ReleaseType
        {
            get { return releaseType; }
        }

        ///////////////////////////////////////////////////////////////////////

        private SignatureFlags signatureFlags;
        public SignatureFlags SignatureFlags
        {
            get { return signatureFlags; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string coreDirectory;
        public string CoreDirectory
        {
            get { return coreDirectory; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string coreFileName;
        public string CoreFileName
        {
            get { return coreFileName; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string hashAlgorithm;
        public string HashAlgorithm
        {
            get { return hashAlgorithm; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string commandFormat;
        public string CommandFormat
        {
            get { return commandFormat; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string argumentFormat;
        public string ArgumentFormat
        {
            get { return argumentFormat; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string logFileName;
        public string LogFileName
        {
            get { return logFileName; }
        }

        ///////////////////////////////////////////////////////////////////////

        private TraceCallback traceCallback;
        public TraceCallback TraceCallback
        {
            get { return traceCallback; }
            set { traceCallback = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string[] shellArgs;
        public string[] ShellArgs
        {
            get { return shellArgs; }
        }

        ///////////////////////////////////////////////////////////////////////

        private bool coreIsAssembly;
        public bool CoreIsAssembly
        {
            get { return coreIsAssembly; }
        }

        ///////////////////////////////////////////////////////////////////////

        private bool whatIf;
        public bool WhatIf
        {
            get { return whatIf; }
        }

        ///////////////////////////////////////////////////////////////////////

        private bool verbose;
        public bool Verbose
        {
            get { return verbose; }
        }

        ///////////////////////////////////////////////////////////////////////

        private bool silent;
        public bool Silent
        {
            get { return silent; }
        }

        ///////////////////////////////////////////////////////////////////////

        private bool invisible;
        public bool Invisible
        {
            get { return invisible; }
        }

        ///////////////////////////////////////////////////////////////////////

        private bool force;
        public bool Force
        {
            get { return force; }
        }

        ///////////////////////////////////////////////////////////////////////

        private bool tracing;
        public bool Tracing
        {
            get { return tracing; }
        }

        ///////////////////////////////////////////////////////////////////////

        private bool logging;
        public bool Logging
        {
            get { return logging; }
        }

        ///////////////////////////////////////////////////////////////////////

        private bool shell;
        public bool Shell
        {
            get { return shell; }
        }

        ///////////////////////////////////////////////////////////////////////

        public bool IsSigned
        {
            get { return isAuthenticodeSigned && isStrongNameSigned; }
        }

        ///////////////////////////////////////////////////////////////////////

        public bool IsValid
        {
            get
            {
                try
                {
                    if (assembly == null)
                        return false;

                    if (String.IsNullOrEmpty(subjectName))
                        return false;

#if !DEBUG
                    if (certificate2 == null)
                        return false;
#endif

                    if (protocolId == null)
                        return false;

                    if (publicKeyToken == null)
                        return false;

                    //if (delay < 0) // NOTE: Negative means "no delay".
                    //    return false;

                    if (mutexName == null)
                        return false;

                    if (baseUri == null)
                        return false;

                    if (String.IsNullOrEmpty(tagFileName))
                        return false;

                    if (uriFormat == null) // NOTE: Empty allowed.
                        return false;

                    if (String.IsNullOrEmpty(name))
                        return false;

                    if (culture == null)
                        return false;

                    if (patchLevel == null)
                        return false;

                    if ((buildType == BuildType.None) ||
                        (buildType == BuildType.Invalid))
                    {
                        return false;
                    }

                    if ((releaseType == ReleaseType.None) ||
                        (releaseType == ReleaseType.Invalid))
                    {
                        return false;
                    }

                    //
                    // NOTE: The value "None" is allowed here; however, it is
                    //       not advised.
                    //
                    if (HasFlags(signatureFlags, SignatureFlags.Invalid, false))
                        return false;

                    if (String.IsNullOrEmpty(coreDirectory) ||
                        !Directory.Exists(coreDirectory))
                    {
                        return false;
                    }

                    if (String.IsNullOrEmpty(coreFileName) ||
                        !File.Exists(coreFileName))
                    {
                        return false;
                    }

                    if (String.IsNullOrEmpty(hashAlgorithm))
                    {
                        return false;
                    }

                    if (String.IsNullOrEmpty(commandFormat))
                        return false;

                    if (argumentFormat == null) // NOTE: Empty allowed.
                        return false;

                    if (String.IsNullOrEmpty(logFileName))
                        return false;

                    if (traceCallback == null)
                        return false;

                    //if (shellArgs == null) // NOTE: Null allowed.
                    //    return false;

                    return true;
                }
                catch (Exception e)
                {
                    TraceOps.Trace(this, e, TraceCategory);
                }

                return false;
            }
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Private Methods
        private void RefreshCoreFileName()
        {
            //
            // NOTE: First, check the configured core file name.  If it looks
            //       valid, remove the directory name from it to obtain just
            //       the file name itself.  Otherwise, use the default core
            //       file name.  Also use the default core file name if the
            //       Path.GetFileName method somehow returns null or an empty
            //       string.
            //
            string fileName = coreFileName;

            if (!String.IsNullOrEmpty(fileName))
                fileName = Path.GetFileName(fileName);

            if (String.IsNullOrEmpty(fileName))
                fileName = Defaults.CoreFileName;

            //
            // NOTE: *SPECIAL* Rebuild the core file name based on the old
            //       base file name and the new (?) core directory.
            //
            coreFileName = (coreDirectory != null) ?
                Path.Combine(coreDirectory, fileName) : fileName;

            //
            // NOTE: *SPECIAL* Must refresh the patch level here because the
            //       underlying core file name has changed.
            //
            RefreshCorePatchLevel();
        }

        ///////////////////////////////////////////////////////////////////////

        private void RefreshCorePatchLevel()
        {
            //
            // NOTE: Refresh the patch level for the core file name.  If this
            //       fails, the new value will be null.
            //
            patchLevel = FileOps.GetVersion(this, coreFileName);
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Public Methods
        public bool HasFlags(
            SignatureFlags hasFlags,
            bool all
            )
        {
            return HasFlags(signatureFlags, hasFlags, all);
        }

        ///////////////////////////////////////////////////////////////////////

        public void ResetCoreDirectory()
        {
            //
            // NOTE: Based on the configured assembly, set the core directory
            //       and refresh the core file name and patch level, as
            //       necessary.  If the configured assembly is null, the core
            //       directory will be as well.
            //
            if (assembly != null)
                SetCoreDirectory(Path.GetDirectoryName(assembly.Location));
            else
                SetCoreDirectory(null);
        }

        ///////////////////////////////////////////////////////////////////////

        public void SetCoreDirectory(
            string directory
            )
        {
            //
            // NOTE: *SPECIAL* Must refresh the core file name and patch level
            //       here because the underlying core directory has changed.
            //
            coreDirectory = directory; RefreshCoreFileName();
        }

        ///////////////////////////////////////////////////////////////////////

        public void SetCoreFileName(
            string fileName
            )
        {
            //
            // NOTE: *SPECIAL* Must refresh the patch level here because the
            //       underlying core file name has changed.
            //
            coreFileName = fileName; RefreshCorePatchLevel();
        }

        ///////////////////////////////////////////////////////////////////////

        public void Dump()
        {
            if (traceCallback != null)
            {
                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Assembly", assembly), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "SubjectName", subjectName), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Certificate2", certificate2), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "ProtocolId", protocolId), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "PublicKeyToken", FormatOps.ToHexString(
                        publicKeyToken)), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Delay", delay), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "MutexName", mutexName), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "BaseUri", baseUri), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "TagFileName", tagFileName), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "UriFormat", uriFormat), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Name", name), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Culture", FormatOps.CultureToString(culture)),
                    TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "PatchLevel", patchLevel), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "BuildType", buildType), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "ReleaseType", releaseType), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "SignatureFlags", signatureFlags), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "CoreDirectory", coreDirectory), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "CoreFileName", coreFileName), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "HashAlgorithm", hashAlgorithm), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "CommandFormat", commandFormat), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "ArgumentFormat", argumentFormat), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "LogFileName", logFileName), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "TraceCallback ", FormatOps.DelegateToString(
                        traceCallback)), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "ShellArgs", FormatOps.ListToString(
                        shellArgs)), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "CoreIsAssembly", coreIsAssembly), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "WhatIf", whatIf), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Verbose", verbose), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Silent", silent), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Invisible", invisible), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Force", force), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Tracing", tracing), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Logging", logging), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "Shell", shell), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "IsAuthenticodeSigned", isAuthenticodeSigned), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "IsStrongNameSigned", isStrongNameSigned), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "IsSigned", IsSigned), TraceCategory);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "IsValid", IsValid), TraceCategory);
            }
        }

        ///////////////////////////////////////////////////////////////////////

        public void Dump(
            Assembly assembly
            )
        {
            if (traceCallback != null)
            {
                string release = AttributeOps.GetAssemblyRelease(assembly);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "AssemblyRelease", release), TraceCategory);

                string sourceId = AttributeOps.GetAssemblySourceId(assembly);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "AssemblySourceId", sourceId), TraceCategory);

                string sourceTimeStamp =
                    AttributeOps.GetAssemblySourceTimeStamp(assembly);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "AssemblySourceTimeStamp", sourceTimeStamp), TraceCategory);

                string tag = AttributeOps.GetAssemblyTag(assembly);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "AssemblyTag", tag), TraceCategory);

                string text = AttributeOps.GetAssemblyText(assembly);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "AssemblyText", text), TraceCategory);

                string title = AttributeOps.GetAssemblyTitle(assembly);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "AssemblyTitle", title), TraceCategory);

                Uri uri = AttributeOps.GetAssemblyUri(assembly);

                traceCallback(String.Format(FormatOps.NameAndValueFormat,
                    "AssemblyUri", uri), TraceCategory);
            }
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Private Static Methods
        private static bool HasFlags(
            SignatureFlags flags,
            SignatureFlags hasFlags,
            bool all
            )
        {
            if (all)
                return ((flags & hasFlags) == hasFlags);
            else
                return ((flags & hasFlags) != SignatureFlags.None);
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Public Static Methods
        public static Configuration CreateDefault()
        {
            return new Configuration(
                Defaults.Assembly, Defaults.SubjectName, null,
                Defaults.ProtocolId, null, Defaults.Delay, Defaults.MutexName,
                null, Defaults.TagFileName, Defaults.UriFormat, Defaults.Name,
                Defaults.Culture, Defaults.PatchLevel, Defaults.BuildType,
                Defaults.ReleaseType, Defaults.SignatureFlags, null,
                Defaults.CoreFileName, Defaults.HashAlgorithm,
                Defaults.CommandFormat, Defaults.ArgumentFormat, null,
                Defaults.TraceCallback, Defaults.ShellArgs,
                Defaults.CoreIsAssembly, Defaults.WhatIf, Defaults.Verbose,
                Defaults.Silent, Defaults.Invisible, Defaults.Force,
                Defaults.Tracing, Defaults.Logging, Defaults.Shell);
        }

        ///////////////////////////////////////////////////////////////////////

        public static Configuration CreateFrom(
            Release release
            )
        {
            if (release != null)
            {
                return new Configuration(
                    Defaults.Assembly, Defaults.SubjectName, null,
                    release.ProtocolId, release.PublicKeyToken, Defaults.Delay,
                    Defaults.MutexName, release.BaseUri, Defaults.TagFileName,
                    Defaults.UriFormat, release.Name, release.Culture,
                    release.PatchLevel, Defaults.BuildType,
                    Defaults.ReleaseType, Defaults.SignatureFlags, null,
                    Defaults.CoreFileName, Defaults.HashAlgorithm,
                    Defaults.CommandFormat, Defaults.ArgumentFormat, null,
                    Defaults.TraceCallback, Defaults.ShellArgs,
                    Defaults.CoreIsAssembly, Defaults.WhatIf, Defaults.Verbose,
                    Defaults.Silent, Defaults.Invisible, Defaults.Force,
                    Defaults.Tracing, Defaults.Logging, Defaults.Shell);
            }

            return CreateDefault();
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool TryCreate(
            Assembly assembly,
            ref Configuration configuration,
            ref string error
            )
        {
            try
            {
                #region Parameter Validation
                if (assembly == null)
                {
                    error = "Invalid assembly.";
                    return false;
                }
                #endregion

                ///////////////////////////////////////////////////////////////

                #region Create Default Configuration
                if (configuration == null)
                    configuration = CreateDefault();
                #endregion

                ///////////////////////////////////////////////////////////////

                #region Store Assembly
                //
                // NOTE: Store the assembly provided by our caller.  This must
                //       be done prior to calling the ResetCoreDirectory method
                //       (below) or things will not be configured correctly.
                //
                configuration.assembly = assembly;
                #endregion

                ///////////////////////////////////////////////////////////////

                #region Setup Default Logging
                configuration.logFileName = FileOps.GetLogName(
                    configuration, String.Format("{0}{1}",
                    VersionOps.GetAssemblyName(assembly),
                    Characters.Period));
                #endregion

                ///////////////////////////////////////////////////////////////

                #region Setup Default Base URI
                Uri baseUri = AttributeOps.GetAssemblyUri(assembly);

                if (baseUri == null)
                    baseUri = new Uri(Defaults.BaseUri);

                configuration.baseUri = baseUri;
                #endregion

                ///////////////////////////////////////////////////////////////

                #region Setup Default Public Key Token
                configuration.publicKeyToken =
                    ParseOps.HexString(Defaults.PublicKeyToken);
                #endregion

                ///////////////////////////////////////////////////////////////

                #region Setup Default Core Directory / File Name / Version
                //
                // NOTE: Reset to the "default" core directory based on the
                //       location of the configured assembly (i.e. the one
                //       as originally provided by our caller).
                //
                configuration.ResetCoreDirectory();
                #endregion

                ///////////////////////////////////////////////////////////////

                #region Setup Default Build Type and Release Type
                //
                // NOTE: Grab the "assembly text" for this assembly.
                //       By convention [only], this is assumed to be
                //       the build type [and release type] this tool
                //       was compiled for.
                //
                string text = AttributeOps.GetAssemblyText(assembly);

                if (!String.IsNullOrEmpty(text))
                {
                    if (!ParseOps.BuildTypeAndReleaseType(
                            text, false, true,
                            ref configuration.buildType,
                            ref configuration.releaseType))
                    {
                        TraceOps.Trace(configuration, String.Format(
                            "Invalid assembly build/release type value: {0}",
                            FormatOps.ForDisplay(text)), TraceCategory);

                        return false;
                    }
                }
                #endregion

                ///////////////////////////////////////////////////////////////

                #region Setup Default What-If Mode
                //
                // NOTE: Never actually modify files when running in the
                //       IDE (or under another debugger).
                //
                if (Debugger.IsAttached)
                {
                    TraceOps.Trace(configuration,
                        "Debugger attached, enabling \"what-if\" mode...",
                        TraceCategory);

                    configuration.whatIf = true;
                }
                #endregion

                ///////////////////////////////////////////////////////////////

                return true;
            }
            catch (Exception e)
            {
                TraceOps.Trace(configuration, e, TraceCategory);

                error = "Failed to create configuration.";
            }

            return false;
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool FromArgs(
            string[] args,
            bool strict,
            ref Configuration configuration,
            ref string error
            )
        {
            try
            {
                if (args == null)
                    return true;

                if (configuration == null)
                    configuration = CreateDefault();

                int length = args.Length;

                for (int index = 0; index < length; index++)
                {
                    string arg = args[index];

                    if (String.IsNullOrEmpty(arg))
                        continue;

                    string newArg = arg;

                    if (ParseOps.CheckOption(ref newArg))
                    {
                        //
                        // NOTE: All the supported command line options must
                        //       have a value; therefore, attempt to advance
                        //       to it now.  If we fail, we are done.
                        //
                        index++;

                        if (index >= length)
                        {
                            error = TraceOps.Trace(
                                configuration, String.Format(
                                "Missing value for option: {0}",
                                FormatOps.ForDisplay(arg)),
                                TraceCategory);

                            if (strict)
                                return false;

                            break;
                        }

                        //
                        // NOTE: Grab the textual value of this command line
                        //       option.
                        //
                        string text = args[index];

                        //
                        // NOTE: Figure out which command line option this is
                        //       (based on a partial name match) and then try
                        //       to interpret the textual value as the correct
                        //       type.
                        //
                        if (ParseOps.MatchOption(newArg, "strict"))
                        {
                            bool? value = ParseOps.Boolean(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} boolean value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            //
                            // NOTE: Allow the command line arguments to override
                            //       the "strictness" setting provided by our caller.
                            //
                            strict = (bool)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "subjectName"))
                        {
                            configuration.subjectName = text;
                        }
                        else if (ParseOps.MatchOption(newArg, "protocolId"))
                        {
                            configuration.protocolId = text;
                        }
                        else if (ParseOps.MatchOption(newArg, "publicKeyToken"))
                        {
                            byte[] value = ParseOps.HexString(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid public key token value: {0}",
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.publicKeyToken = value;
                        }
                        else if (ParseOps.MatchOption(newArg, "delay"))
                        {
                            int? value = ParseOps.Integer(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} integer value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.delay = (int)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "mutexName"))
                        {
                            configuration.mutexName = text;
                        }
                        else if (ParseOps.MatchOption(newArg, "baseUri"))
                        {
                            Uri value = ParseOps.Uri(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} URI value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.baseUri = value;
                        }
                        else if (ParseOps.MatchOption(newArg, "tagFileName"))
                        {
                            configuration.tagFileName = text;
                        }
                        else if (ParseOps.MatchOption(newArg, "uriFormat"))
                        {
                            configuration.uriFormat = text;
                        }
                        else if (ParseOps.MatchOption(newArg, "name"))
                        {
                            configuration.name = text;
                        }
                        else if (ParseOps.MatchOption(newArg, "culture"))
                        {
                            CultureInfo value = ParseOps.Culture(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid culture value: {0}",
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.culture = value;
                        }
                        else if (ParseOps.MatchOption(newArg, "patchLevel"))
                        {
                            Version value = ParseOps.Version(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} version value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.patchLevel = value;
                        }
                        else if (ParseOps.MatchOption(newArg, "buildType"))
                        {
                            object value = ParseOps.Enum(
                                typeof(BuildType), text, true);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid build type value: {0}",
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.buildType = (BuildType)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "releaseType"))
                        {
                            object value = ParseOps.Enum(
                                typeof(ReleaseType), text, true);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid release type value: {0}",
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.releaseType = (ReleaseType)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "signatureFlags"))
                        {
                            object value = ParseOps.Enum(
                                typeof(SignatureFlags), text, true);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid signature flags value: {0}",
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.signatureFlags = (SignatureFlags)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "coreDirectory"))
                        {
                            configuration.SetCoreDirectory(text);
                        }
                        else if (ParseOps.MatchOption(newArg, "coreFileName"))
                        {
                            configuration.SetCoreFileName(text);
                        }
                        else if (ParseOps.MatchOption(newArg, "hashAlgorithm"))
                        {
                            configuration.hashAlgorithm = text;
                        }
                        else if (ParseOps.MatchOption(newArg, "commandFormat"))
                        {
                            configuration.commandFormat = text;
                        }
                        else if (ParseOps.MatchOption(newArg, "argumentFormat"))
                        {
                            configuration.argumentFormat = text;
                        }
                        else if (ParseOps.MatchOption(newArg, "logFileName"))
                        {
                            configuration.logFileName = text;
                        }
                        else if (ParseOps.MatchOption(newArg, "coreIsAssembly"))
                        {
                            bool? value = ParseOps.Boolean(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} boolean value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.coreIsAssembly = (bool)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "whatIf"))
                        {
                            bool? value = ParseOps.Boolean(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} boolean value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.whatIf = (bool)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "verbose"))
                        {
                            bool? value = ParseOps.Boolean(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} boolean value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.verbose = (bool)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "silent"))
                        {
                            bool? value = ParseOps.Boolean(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} boolean value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.silent = (bool)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "invisible"))
                        {
                            bool? value = ParseOps.Boolean(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} boolean value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.invisible = (bool)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "force"))
                        {
                            bool? value = ParseOps.Boolean(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} boolean value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.force = (bool)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "tracing"))
                        {
                            bool? value = ParseOps.Boolean(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} boolean value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.tracing = (bool)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "logging"))
                        {
                            bool? value = ParseOps.Boolean(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} boolean value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.logging = (bool)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "shell"))
                        {
                            bool? value = ParseOps.Boolean(text);

                            if (value == null)
                            {
                                error = TraceOps.Trace(
                                    configuration, String.Format(
                                    "Invalid {0} boolean value: {1}",
                                    FormatOps.ForDisplay(arg),
                                    FormatOps.ForDisplay(text)),
                                    TraceCategory);

                                if (strict)
                                    return false;

                                continue;
                            }

                            configuration.shell = (bool)value;
                        }
                        else if (ParseOps.MatchOption(newArg, "shellArgs"))
                        {
                            configuration.shellArgs = ParseOps.CommandLine(text);
                        }
                        else
                        {
                            error = TraceOps.Trace(
                                configuration, String.Format(
                                "Unsupported command line option: {0}",
                                FormatOps.ForDisplay(arg)),
                                TraceCategory);

                            if (strict)
                                return false;
                        }
                    }
                    else
                    {
                        error = TraceOps.Trace(
                            configuration, String.Format(
                            "Unsupported command line argument: {0}",
                            FormatOps.ForDisplay(arg)),
                            TraceCategory);

                        if (strict)
                            return false;
                    }
                }

                return true;
            }
            catch (Exception e)
            {
                TraceOps.Trace(configuration, e, TraceCategory);

                error = "Failed to modify configuration.";
            }

            return false;
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool Process(
            string[] args,
            Configuration configuration,
            bool strict,
            ref string error
            )
        {
            try
            {
                if (configuration == null)
                {
                    error = "Invalid configuration.";
                    return false;
                }

                Assembly assembly = configuration.assembly;

                if (assembly == null)
                {
                    error = "Invalid assembly.";
                    return false;
                }

                //
                // HACK: If tracing has been enabled, add the listener.
                //
                if (configuration.tracing)
                    Trace.Listeners.Add(new ConsoleTraceListener());

                //
                // NOTE: If logging has been enabled, add the listener.
                //
                if (configuration.logging)
                    Trace.Listeners.Add(new TextWriterTraceListener(
                        configuration.logFileName));

                string localError = null;

                //
                // NOTE: Grab the Authenticode signature of the assembly
                //       and use it to populate the configuration.
                //
                if (configuration.HasFlags(SignatureFlags.Self, true))
                {
                    X509Certificate2 certificate2 = null;

                    if (SecurityOps.IsAuthenticodeSigned(
                            assembly, configuration.subjectName, false,
                            ref certificate2, ref localError)
#if NATIVE && WINDOWS
                        && (!VersionOps.IsWindowsOperatingSystem()
                        || WinTrustEx.IsFileTrusted(assembly.Location,
                            !configuration.invisible, false, true, false,
                            ref localError))
#endif
                        )
                    {
                        configuration.isAuthenticodeSigned = true;
                    }
                    else if (strict)
                    {
                        error = localError;
                        return false;
                    }
                    else
                    {
                        TraceOps.Trace(configuration, localError, TraceCategory);
                    }

                    configuration.certificate2 = certificate2;
                }

                //
                // NOTE: Grab the public key token for the assembly [and
                //       possibly] use it to populate the configuration.
                //
                byte[] publicKeyToken = null;

                if (SecurityOps.IsStrongNameSigned(assembly,
                        ref publicKeyToken, ref localError)
#if NATIVE && WINDOWS
                    && (!VersionOps.IsWindowsOperatingSystem()
                    || StrongNameEx.IsStrongNameSigned(
                        assembly.Location, true, ref localError))
#endif
                    )
                {
                    configuration.isStrongNameSigned = true;
                }
                else if (strict)
                {
                    error = localError;
                    return false;
                }
                else
                {
                    TraceOps.Trace(configuration, localError, TraceCategory);
                }

                //
                // NOTE: If the public key token that the assembly is
                //       signed with differs from our default, use it
                //       unless the configuration has previously been
                //       modified to use a non-default public key token.
                //
                if (!SecurityOps.IsDefaultPublicKeyToken(publicKeyToken) &&
                    SecurityOps.IsDefaultPublicKeyToken(configuration.publicKeyToken))
                {
                    TraceOps.Trace(configuration, String.Format(
                        "Using non-default public key token: \"{0}\"...",
                        FormatOps.ToHexString(publicKeyToken)), TraceCategory);

                    configuration.publicKeyToken = publicKeyToken;
                }

                //
                // NOTE: If the culture is null, use the invariant culture.
                //
                if (configuration.culture == null)
                    configuration.culture = CultureInfo.InvariantCulture;

                //
                // NOTE: Dump the configuration now in case we need to
                //       troubleshoot any issues.
                //
                configuration.Dump(assembly);
                configuration.Dump();

#if NATIVE && WINDOWS
                //
                // NOTE: If we are running on Windows, always try to close
                //       our console window.  If the shell has been enabled,
                //       it will try to re-open the console window later
                //       (i.e. if it is actually invoked).
                //
                if (VersionOps.IsWindowsOperatingSystem() &&
                    !ConsoleEx.TryClose(ref error))
                {
                    return false;
                }
#endif

                //
                // NOTE: Add an entry to the log, if applicable.
                //
                TraceOps.Trace(configuration, String.Format(
                    "Configuration processed for assembly \"{0}\".",
                    assembly), TraceCategory);

                //
                // NOTE: Show where we are running from and how we were
                //       invoked.
                //
                string location = assembly.Location;

                TraceOps.Trace(configuration, String.Format(
                    "Original command line is: {0}", Environment.CommandLine),
                    TraceCategory);

                TraceOps.Trace(configuration, String.Format(
                    "Running as \"{0}\" with arguments \"{1}\".",
                    location, FormatOps.ListToString(args)), TraceCategory);

                //
                // NOTE: If the debugger is attached and What-If mode is [now]
                //       disabled, issue a warning.
                //
                if (!configuration.whatIf && Debugger.IsAttached)
                {
                    TraceOps.Trace(configuration,
                        "Forced to disable \"what-if\" mode with debugger " +
                        "attached.", TraceCategory);
                }

                return true;
            }
            catch (Exception e)
            {
                TraceOps.Trace(configuration, e, TraceCategory);

                error = "Failed to process configuration.";
            }

            return false;
        }
        #endregion
    }
}
